@@ -38,21 +38,22 @@ exception statement from your version. */
 
 package java.awt;
 
-import java.io.Serializable;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.awt.peer.ListPeer;
-import java.awt.peer.ComponentPeer;
+import java.util.EventListener;
 import java.util.Vector;
+import javax.accessibility.Accessible;
 
 /**
   * Class that implements a listbox widget
   *
   * @author Aaron M. Renn (arenn@urbanophile.com)
   */
-public class List extends Component implements ItemSelectable, Serializable
+public class List extends Component
+  implements ItemSelectable, Accessible
 {
 
 /*
@@ -113,6 +114,8 @@ private ActionListener action_listeners;
 /**
   * Initializes a new instance of <code>List</code> with no visible lines
   * and multi-select disabled.
+  *
+  * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
   */
 public
 List()
@@ -127,6 +130,8 @@ List()
   * number of visible lines and multi-select disabled.
   *
   * @param lines The number of visible lines in the list.
+  *
+  * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
   */
 public
 List(int rows)
@@ -143,12 +148,17 @@ List(int rows)
   * @param lines The number of visible lines in the list.
   * @param multipleMode <code>true</code> if multiple lines can be selected
   * simultaneously, <code>false</code> otherwise.
+  *
+  * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
   */
 public 
 List(int rows, boolean multipleMode)
 {
   this.rows = rows;
   this.multipleMode = multipleMode;
+
+  if (GraphicsEnvironment.isHeadless())
+    throw new HeadlessException ();
 }
 
 /*************************************************************************/
@@ -287,6 +297,8 @@ setMultipleMode(boolean multipleMode)
   *
   * @param multipleMode <code>true</code> to enable multiple mode,
   * <code>false</code> otherwise.
+  *
+  * @deprecated
   */
 public void
 setMultipleSelections(boolean multipleMode)
@@ -511,6 +523,8 @@ addItem(String item, int index)
   * @param index The index of the item to delete.
   *
   * @exception IllegalArgumentException If the index is not valid
+  *
+  * @deprecated
   */
 public void
 delItem(int index) throws IllegalArgumentException
@@ -633,8 +647,21 @@ clear()
 public synchronized void
 replaceItem(String item, int index) throws IllegalArgumentException
 {
-  remove(index);
-  addItem(item, index);
+  if ((index < 0) || (index >= items.size()))
+    throw new IllegalArgumentException("Bad list index: " + index);
+
+  items.insertElementAt(item, index + 1);
+  items.removeElementAt (index);
+
+  if (peer != null)
+    {
+      ListPeer l = (ListPeer) peer;
+
+      /* We add first and then remove so that the selected
+	 item remains the same */
+      l.add (item, index + 1);
+      l.delItems (index, index);
+    }
 }
 
 /*************************************************************************/
@@ -654,7 +681,7 @@ getSelectedIndex()
       selected = l.getSelectedIndexes ();
     }
 
-  if (selected == null || selected.length > 1)
+  if (selected == null || selected.length != 1)
     return -1;
   return selected[0];
 }
@@ -947,10 +974,10 @@ processEvent(AWTEvent event)
 {
   if (event instanceof ActionEvent)
     processActionEvent((ActionEvent)event);
-  if (event instanceof ItemEvent)
+  else if (event instanceof ItemEvent)
     processItemEvent((ItemEvent)event);
-
-  super.processEvent(event);
+  else
+    super.processEvent(event);
 }
 
 /*************************************************************************/
@@ -989,6 +1016,23 @@ processItemEvent(ItemEvent event)
     item_listeners.itemStateChanged(event);
 }
 
+void
+dispatchEventImpl(AWTEvent e)
+{
+  if (e.id <= ItemEvent.ITEM_LAST
+      && e.id >= ItemEvent.ITEM_FIRST
+      && (item_listeners != null 
+	  || (eventMask & AWTEvent.ITEM_EVENT_MASK) != 0))
+    processEvent(e);
+  else if (e.id <= ActionEvent.ACTION_LAST 
+	   && e.id >= ActionEvent.ACTION_FIRST
+	   && (action_listeners != null 
+	       || (eventMask & AWTEvent.ACTION_EVENT_MASK) != 0))
+    processEvent(e);
+  else
+    super.dispatchEventImpl(e);
+}
+
 /*************************************************************************/
 
 /**
@@ -1002,4 +1046,38 @@ paramString()
   return "multiple=" + multipleMode + ",rows=" + rows + super.paramString();
 }
 
+  /**
+   * Returns an array of all the objects currently registered as FooListeners
+   * upon this <code>List</code>. FooListeners are registered using the 
+   * addFooListener method.
+   *
+   * @exception ClassCastException If listenerType doesn't specify a class or
+   * interface that implements java.util.EventListener.
+   */
+  public EventListener[] getListeners (Class listenerType)
+  {
+    if (listenerType == ActionListener.class)
+      return AWTEventMulticaster.getListeners (action_listeners, listenerType);
+    
+    if (listenerType == ItemListener.class)
+      return AWTEventMulticaster.getListeners (item_listeners, listenerType);
+
+    return super.getListeners (listenerType);
+  }
+
+  /**
+   * Returns all action listeners registered to this object.
+   */
+  public ActionListener[] getActionListeners ()
+  {
+    return (ActionListener[]) getListeners (ActionListener.class);
+  }
+  
+  /**
+   * Returns all action listeners registered to this object.
+   */
+  public ItemListener[] getItemListeners ()
+  {
+    return (ItemListener[]) getListeners (ItemListener.class);
+  }
 } // class List
