@@ -1,5 +1,5 @@
 /* Toolkit.java -- AWT Toolkit superclass
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -96,7 +96,7 @@ public abstract class Toolkit
 {
   /** The default toolkit name. */
   private static String default_toolkit_name
-    = "gnu.awt.gtk.GtkToolkit";
+    = gnu.classpath.Configuration.default_awt_peer_toolkit;
 
   /**
    * The toolkit in use.  Once we load it, we don't ever change it
@@ -134,6 +134,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>TextField</code>.
    *
    * @param target The <code>TextField</code> to create the peer for.
+   *
    * @return The peer for the specified <code>TextField</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -144,6 +145,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Label</code>.
    *
    * @param target The <code>Label</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Label</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -154,6 +156,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>List</code>.
    *
    * @param target The <code>List</code> to create the peer for.
+   *
    * @return The peer for the specified <code>List</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -164,6 +167,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Checkbox</code>.
    *
    * @param target The <code>Checkbox</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Checkbox</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -174,6 +178,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Scrollbar</code>.
    *
    * @param target The <code>Scrollbar</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Scrollbar</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -184,6 +189,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>ScrollPane</code>.
    *
    * @param target The <code>ScrollPane</code> to create the peer for.
+   *
    * @return The peer for the specified <code>ScrollPane</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -194,6 +200,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>TextArea</code>.
    *
    * @param target The <code>TextArea</code> to create the peer for.
+   *
    * @return The peer for the specified <code>TextArea</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -204,6 +211,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Choice</code>.
    *
    * @param target The <code>Choice</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Choice</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -214,6 +222,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Frame</code>.
    *
    * @param target The <code>Frame</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Frame</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -224,6 +233,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Canvas</code>.
    *
    * @param target The <code>Canvas</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Canvas</code> object.
    */
   protected abstract CanvasPeer createCanvas(Canvas target);
@@ -232,6 +242,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Panel</code>.
    *
    * @param target The <code>Panel</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Panel</code> object.
    */
   protected abstract PanelPeer createPanel(Panel target);
@@ -240,6 +251,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Window</code>.
    *
    * @param target The <code>Window</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Window</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -250,6 +262,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Dialog</code>.
    *
    * @param target The dialog to create the peer for
+   *
    * @return The peer for the specified font name.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -260,6 +273,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>MenuBar</code>.
    *
    * @param target The <code>MenuBar</code> to create the peer for.
+   *
    * @return The peer for the specified <code>MenuBar</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -270,6 +284,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>Menu</code>.
    *
    * @param target The <code>Menu</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Menu</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -280,6 +295,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>PopupMenu</code>.
    *
    * @param target The <code>PopupMenu</code> to create the peer for.
+   *
    * @return The peer for the specified <code>PopupMenu</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -290,6 +306,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>MenuItem</code>.
    *
    * @param target The <code>MenuItem</code> to create the peer for.
+   *
    * @return The peer for the specified <code>MenuItem</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -300,6 +317,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>FileDialog</code>.
    *
    * @param target The <code>FileDialog</code> to create the peer for.
+   *
    * @return The peer for the specified <code>FileDialog</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -310,6 +328,7 @@ public abstract class Toolkit
    * Creates a peer object for the specified <code>CheckboxMenuItem</code>.
    *
    * @param target The <code>CheckboxMenuItem</code> to create the peer for.
+   *
    * @return The peer for the specified <code>CheckboxMenuItem</code> object.
    *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
@@ -323,14 +342,13 @@ public abstract class Toolkit
    * with its own native window.  Instead, this method allows the component
    * to draw on its parent window as a "lightweight" widget.
    *
-   * XXX: FIXME
-   *
    * @param target The <code>Component</code> to create the peer for.
+   *
    * @return The peer for the specified <code>Component</code> object.
    */
   protected LightweightPeer createComponent(Component target)
   {
-    return null;
+    return new gnu.java.awt.peer.GLightweightPeer (target);
   }
 
   /**
@@ -338,7 +356,10 @@ public abstract class Toolkit
    *
    * @param name The font to create the peer for.
    * @param style The font style to create the peer for.
+   *
    * @return The peer for the specified font name.
+   *
+   * @deprecated
    */
   protected abstract FontPeer getFontPeer(String name, int style);
 
@@ -416,6 +437,8 @@ public abstract class Toolkit
    * Returns the color model of the screen.
    *
    * @return The color model of the screen.
+   * 
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
    */
   public abstract ColorModel getColorModel();
 
@@ -423,6 +446,8 @@ public abstract class Toolkit
    * Returns the names of the available fonts.
    *
    * @return The names of the available fonts.
+   *
+   * @deprecated
    */
   public abstract String[] getFontList();
 
@@ -430,7 +455,10 @@ public abstract class Toolkit
    * Return the font metrics for the specified font
    *
    * @param name The name of the font to return metrics for.
+   *
    * @return The requested font metrics.
+   *
+   * @deprecated
    */
   public abstract FontMetrics getFontMetrics(Font name);
 
@@ -466,9 +494,10 @@ public abstract class Toolkit
         toolkit = (Toolkit) obj;
         return toolkit;
       }
-    catch (Exception e)
+    catch (Throwable t)
       {
-        throw new AWTError("Cannot load AWT toolkit: " + e.getMessage());
+	AWTError e = new AWTError("Cannot load AWT toolkit: " + toolkit_name);
+	throw (AWTError) e.initCause(t);
       }
   }
 
@@ -571,12 +600,32 @@ public abstract class Toolkit
    *
    * @return The requested print job, or <code>null</code> if the job
    * was cancelled.
+   *
+   * @exception NullPointerException If frame is null,
+   * or GraphicsEnvironment.isHeadless() returns true.
+   * @exception SecurityException If this thread is not allowed to initiate
+   * a print job request.
    */
   public abstract PrintJob getPrintJob(Frame frame, String title,
                                        Properties props);
 
-
   /**
+   * Returns a instance of <code>PrintJob</code> for the specified
+   * arguments.
+   *
+   * @param frame The window initiating the print job.
+   * @param title The print job title.
+   * @param jobAttr A set of job attributes which will control the print job.
+   * @param pageAttr A set of page attributes which will control the print job.
+   *
+   * @exception NullPointerException If frame is null, and either jobAttr is null
+   * or jobAttr.getDialog() returns JobAttributes.DialogType.NATIVE.
+   * @exception IllegalArgumentException If pageAttrspecifies differing cross
+   * feed and feed resolutions, or when GraphicsEnvironment.isHeadless() returns
+   * true.
+   * @exception SecurityException If this thread is not allowed to initiate
+   * a print job request.
+   *
    * @since 1.3
    */
   public PrintJob getPrintJob(Frame frame, String title,
@@ -600,6 +649,8 @@ public abstract class Toolkit
   public abstract Clipboard getSystemClipboard();
 
   /**
+   * Gets the singleton instance of the system selection as a Clipboard object.
+   *
    * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
    *
    * @since 1.4
@@ -623,21 +674,42 @@ public abstract class Toolkit
     return Event.CTRL_MASK;
   }
 
+  /**
+   * Returns whether the given locking key on the keyboard is currently in its
+   * "on" state.
+   *
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   * @exception IllegalArgumentException If keyCode is not one of the valid keys.
+   * @exception UnsupportedOperationException If the host system doesn't allow
+   * getting the state of this key programmatically, or if the keyboard doesn't
+   * have this key.
+   */
   public boolean getLockingKeyState(int keyCode)
   {
     if (keyCode != KeyEvent.VK_CAPS_LOCK
         && keyCode != KeyEvent.VK_NUM_LOCK
         && keyCode != KeyEvent.VK_SCROLL_LOCK)
       throw new IllegalArgumentException();
+    
     throw new UnsupportedOperationException();
   }
 
+  /**
+   * Sets the state of the given locking key on the keyboard.
+   *
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   * @exception IllegalArgumentException If keyCode is not one of the valid keys.
+   * @exception UnsupportedOperationException If the host system doesn't allow
+   * getting the state of this key programmatically, or if the keyboard doesn't
+   * have this key.
+   */
   public void setLockingKeyState(int keyCode, boolean on)
   {
     if (keyCode != KeyEvent.VK_CAPS_LOCK
         && keyCode != KeyEvent.VK_NUM_LOCK
         && keyCode != KeyEvent.VK_SCROLL_LOCK)
       throw new IllegalArgumentException();
+    
     throw new UnsupportedOperationException();
   }
 
@@ -647,6 +719,7 @@ public abstract class Toolkit
    * component.
    *
    * @param component The component to fetch the native container for.
+   *
    * @return The native container object for this component.
    */
   protected static Container getNativeContainer(Component component)
@@ -670,6 +743,13 @@ public abstract class Toolkit
       }
   }
 
+  /**
+   * Creates a new custom cursor object.
+   *
+   * @exception IndexOutOfBoundsException If the hotSpot values are outside
+   * the bounds of the cursor.
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   */
   public Cursor createCustomCursor(Image cursor, Point hotSpot, String name)
   {
     // Presumably the only reason this isn't abstract is for backwards
@@ -677,17 +757,33 @@ public abstract class Toolkit
     return null;
   }
 
+  /**
+   * Returns the supported cursor dimension which is closest to the
+   * desired sizes.
+   *
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   */
   public Dimension getBestCursorSize(int preferredWidth, int preferredHeight)
   {
     return new Dimension (0,0);
   }
 
+  /**
+   * Returns the maximum number of colors the Toolkit supports in a custom
+   * cursor palette.
+   *
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   */
   public int getMaximumCursorColors()
   {
     return 0;
   }
 
   /**
+   * Returns whether Toolkit supports this state for Frames.
+   *
+   * @exception HeadlessException If GraphicsEnvironment.isHeadless() is true.
+   * 
    * @since 1.4
    */
   public boolean isFrameStateSupported(int state)
@@ -700,30 +796,65 @@ public abstract class Toolkit
    * default value if the property does not exist.
    *
    * @param key The name of the property to retrieve.
-   * @param defThe default value of the property.
+   * @param def The default value of the property.
    */
   public static String getProperty(String key, String def)
   {
     return props.getProperty(key, def);
   }
 
+
   /**
-   * Returns the event queue for the applet.  Despite the word "System"
-   * in the name of this method, there is no guarantee that the same queue
-   * is shared system wide.
+   * Returns the event queue that is suitable for the calling context.
    *
-   * @return The event queue for this applet (or application)
+   * <p>Despite the word &#x201c;System&#x201d; in the name of this
+   * method, a toolkit may provide different event queues for each
+   * applet. There is no guarantee that the same queue is shared
+   * system-wide.
+   *
+   * <p>The implementation first checks whether a
+   * SecurityManager has been installed. If so, its {@link
+   * java.lang.SecurityManager#checkAwtEventQueueAccess()} method gets
+   * called. The security manager will throw a SecurityException if it
+   * does not grant the permission to access the event queue.
+   *
+   * <p>Next, the call is delegated to {@link
+   * #getSystemEventQueueImpl()}.
+   *
+   * @return The event queue for this applet (or application).
+   *
+   * @throws SecurityException if a security manager has been
+   * installed, and it does not grant the permission to access the
+   * event queue.
    */
   public final EventQueue getSystemEventQueue()
   {
+    SecurityManager sm;
+
+    sm = System.getSecurityManager();
+    if (sm != null)
+      sm.checkAwtEventQueueAccess();
+
     return getSystemEventQueueImpl();
   }
 
+
   /**
-   * // FIXME: What does this do?
+   * Returns the event queue that is suitable for the calling context.
+   *
+   * <p>Despite the word &#x201c;System&#x201d; in the name of this
+   * method, a toolkit may provide different event queues for each
+   * applet. There is no guarantee that the same queue is shared
+   * system-wide.
+   *
+   * <p>No security checks are performed, which is why this method
+   * may only be called by Toolkits.
+   *
+   * @see #getSystemEventQueue()
    */
   protected abstract EventQueue getSystemEventQueueImpl();
 
+
   /**
    * @since 1.3
    */
