@@ -1,5 +1,6 @@
 /* BufferedReader.java
-   Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003 
+     Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -44,19 +45,19 @@ package java.io;
  */
 
 /**
-  * This subclass of <code>FilterReader</code> buffers input from an 
-  * underlying implementation to provide a possibly more efficient read
-  * mechanism.  It maintains the buffer and buffer state in instance 
-  * variables that are available to subclasses.  The default buffer size
-  * of 512 chars can be overridden by the creator of the stream.
-  * <p>
-  * This class also implements mark/reset functionality.  It is capable
-  * of remembering any number of input chars, to the limits of
-  * system memory or the size of <code>Integer.MAX_VALUE</code>
-  *
-  * @author Per Bothner <bothner@cygnus.com>
-  * @author Aaron M. Renn <arenn@urbanophile.com>
-  */
+ * This subclass of <code>FilterReader</code> buffers input from an 
+ * underlying implementation to provide a possibly more efficient read
+ * mechanism.  It maintains the buffer and buffer state in instance 
+ * variables that are available to subclasses.  The default buffer size
+ * of 8192 chars can be overridden by the creator of the stream.
+ * <p>
+ * This class also implements mark/reset functionality.  It is capable
+ * of remembering any number of input chars, to the limits of
+ * system memory or the size of <code>Integer.MAX_VALUE</code>
+ *
+ * @author Per Bothner <bothner@cygnus.com>
+ * @author Aaron M. Renn <arenn@urbanophile.com>
+ */
 public class BufferedReader extends Reader
 {
   Reader in;
@@ -89,7 +90,7 @@ public class BufferedReader extends Reader
 
   /**
     * Create a new <code>BufferedReader</code> that will read from the 
-    * specified subordinate stream with a default buffer size of 4096 chars.
+    * specified subordinate stream with a default buffer size of 8192 chars.
     *
     * @param in The subordinate stream to read from
     */
@@ -99,25 +100,30 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * Create a new <code>BufferedReader</code> that will read from the 
-    * specified subordinate stream with a buffer size that is specified by the 
-    * caller.
-    *
-    * @param in The subordinate stream to read from
-    * @param bufsize The buffer size to use
-    */
+   * Create a new <code>BufferedReader</code> that will read from the 
+   * specified subordinate stream with a buffer size that is specified by the 
+   * caller.
+   *
+   * @param in The subordinate stream to read from
+   * @param size The buffer size to use
+   *
+   * @exception IllegalArgumentException if size &lt;&eq; 0
+   */
   public BufferedReader(Reader in, int size)
   {
     super(in.lock);
+    if (size <= 0)
+      throw new IllegalArgumentException("Illegal buffer size: " + size);
     this.in = in;
     buffer = new char[size];
   }
 
   /**
-    * This method closes the stream 
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method closes the underlying stream and frees any associated
+   * resources.
+   *
+   * @exception IOException If an error occurs
+   */
   public void close() throws IOException
   {
     synchronized (lock)
@@ -130,38 +136,42 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * Returns <code>true</code> to indicate that this class supports mark/reset 
-    * functionality.
-    *
-    * @return <code>true</code>
-    */
+   * Returns <code>true</code> to indicate that this class supports mark/reset 
+   * functionality.
+   *
+   * @return <code>true</code>
+   */
   public boolean markSupported()
   {
     return true;
   }
 
   /**
-    * Mark a position in the input to which the stream can be
-    * "reset" by calling the <code>reset()</code> method.  The parameter
-    * <code>readlimit</code> is the number of chars that can be read from the 
-    * stream after setting the mark before the mark becomes invalid.  For
-    * example, if <code>mark()</code> is called with a read limit of 10, then 
-    * when 11 chars of data are read from the stream before the 
-    * <code>reset()</code> method is called, then the mark is invalid and the 
-    * stream object instance is not required to remember the mark.
-    * <p>
-    * Note that the number of chars that can be remembered by this method
-    * can be greater than the size of the internal read buffer.  It is also
-    * not dependent on the subordinate stream supporting mark/reset
-    * functionality.
-    *
-    * @param readlimit The number of chars that can be read before the mark 
-    *        becomes invalid
-    *
-    * @exception IOException If an error occurs
-    */
+   * Mark a position in the input to which the stream can be
+   * "reset" by calling the <code>reset()</code> method.  The parameter
+   * <code>readLimit</code> is the number of chars that can be read from the 
+   * stream after setting the mark before the mark becomes invalid.  For
+   * example, if <code>mark()</code> is called with a read limit of 10, then 
+   * when 11 chars of data are read from the stream before the 
+   * <code>reset()</code> method is called, then the mark is invalid and the 
+   * stream object instance is not required to remember the mark.
+   * <p>
+   * Note that the number of chars that can be remembered by this method
+   * can be greater than the size of the internal read buffer.  It is also
+   * not dependent on the subordinate stream supporting mark/reset
+   * functionality.
+   *
+   * @param readLimit The number of chars that can be read before the mark 
+   *        becomes invalid
+   *
+   * @exception IOException If an error occurs
+   * @exception IllegalArgumentException if readLimit is negative.
+   */
   public void mark(int readLimit) throws IOException
   {
+    if (readLimit < 0)
+      throw new IllegalArgumentException("Read-ahead limit is negative");
+
     synchronized (lock)
       {
 	checkStatus();
@@ -207,16 +217,16 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * Reset the stream to the point where the <code>mark()</code> method
-    * was called.  Any chars that were read after the mark point was set will
-    * be re-read during subsequent reads.
-    * <p>
-    * This method will throw an IOException if the number of chars read from
-    * the stream since the call to <code>mark()</code> exceeds the mark limit
-    * passed when establishing the mark.
-    *
-    * @exception IOException If an error occurs;
-    */
+   * Reset the stream to the point where the <code>mark()</code> method
+   * was called.  Any chars that were read after the mark point was set will
+   * be re-read during subsequent reads.
+   * <p>
+   * This method will throw an IOException if the number of chars read from
+   * the stream since the call to <code>mark()</code> exceeds the mark limit
+   * passed when establishing the mark.
+   *
+   * @exception IOException If an error occurs;
+   */
   public void reset() throws IOException
   {
     synchronized (lock)
@@ -239,14 +249,15 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method determines whether or not a stream is ready to be read.  If
-    * This method returns <code>false</code> then this stream could (but is
-    * not guaranteed to) block on the next read attempt.
-    *
-    * @return <code>true</code> if this stream is ready to be read, <code>false</code> otherwise
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method determines whether or not a stream is ready to be read.  If
+   * this method returns <code>false</code> then this stream could (but is
+   * not guaranteed to) block on the next read attempt.
+   *
+   * @return <code>true</code> if this stream is ready to be read, 
+   * <code>false</code> otherwise
+   *
+   * @exception IOException If an error occurs
+   */
   public boolean ready() throws IOException
   {
     synchronized (lock)
@@ -257,25 +268,31 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method read chars from a stream and stores them into a caller
-    * supplied buffer.  It starts storing the data at index <code>offset</code> into
-    * the buffer and attempts to read <code>len</code> chars.  This method can
-    * return before reading the number of chars requested.  The actual number
-    * of chars read is returned as an int.  A -1 is returned to indicate the
-    * end of the stream.
-    * <p>
-    * This method will block until some data can be read.
-    *
-    * @param buf The array into which the chars read should be stored
-    * @param offset The offset into the array to start storing chars
-    * @param count The requested number of chars to read
-    *
-    * @return The actual number of chars read, or -1 if end of stream.
-    *
-    * @exception IOException If an error occurs.
-    */
+   * This method read chars from a stream and stores them into a caller
+   * supplied buffer.  It starts storing the data at index 
+   * <code>offset</code> into
+   * the buffer and attempts to read <code>len</code> chars.  This method can
+   * return before reading the number of chars requested.  The actual number
+   * of chars read is returned as an int.  A -1 is returned to indicate the
+   * end of the stream.
+   * <p>
+   * This method will block until some data can be read.
+   *
+   * @param buf The array into which the chars read should be stored
+   * @param offset The offset into the array to start storing chars
+   * @param count The requested number of chars to read
+   *
+   * @return The actual number of chars read, or -1 if end of stream.
+   *
+   * @exception IOException If an error occurs.
+   * @exception IndexOutOfBoundsException If offset and count are not
+   * valid regarding buf.
+   */
   public int read(char[] buf, int offset, int count) throws IOException
   {
+    if (offset < 0 || offset + count > buf.length || count < 0)
+      throw new IndexOutOfBoundsException();
+
     synchronized (lock)
       {
 	checkStatus();
@@ -394,16 +411,16 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method reads a single line of text from the input stream, returning
-    * it as a <code>String</code>.  A line is terminated by "\n", a "\r", or
-    * an "\r\n" sequence.  The system dependent line separator is not used.
-    * The line termination characters are not returned in the resulting
-    * <code>String</code>.
-    * 
-    * @return The line of text read, or <code>null</code> if end of stream.
-    * 
-    * @exception IOException If an error occurs
-    */
+   * This method reads a single line of text from the input stream, returning
+   * it as a <code>String</code>.  A line is terminated by "\n", a "\r", or
+   * an "\r\n" sequence.  The system dependent line separator is not used.
+   * The line termination characters are not returned in the resulting
+   * <code>String</code>.
+   * 
+   * @return The line of text read, or <code>null</code> if end of stream.
+   * 
+   * @exception IOException If an error occurs
+   */
   public String readLine() throws IOException
   {
     checkStatus();
@@ -443,12 +460,19 @@ public class BufferedReader extends Reader
     boolean eof = false;
     for (;;)
       {
-	int ch = read();
-	if (ch < 0)
+	// readLine should block. So we must not return until a -1 is reached.
+	if (pos >= limit)
 	  {
-	    eof = true;
-	    break;
+	    // here count == 0 isn't sufficient to give a failure.
+	    int count = fill();
+	    if (count < 0)
+	      {
+		eof = true;
+		break;
+	      }
+	    continue;
 	  }
+	int ch = buffer[pos++];
 	if (ch == '\n' || ch == '\r')
 	  {
 	    // Check here if a '\r' was the last char in the buffer; if so,
@@ -468,25 +492,29 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method skips the specified number of chars in the stream.  It
-    * returns the actual number of chars skipped, which may be less than the
-    * requested amount.
-    * <p>
-    * This method first discards chars in the buffer, then calls the
-    * <code>skip</code> method on the underlying stream to skip the remaining chars.
-    *
-    * @param num_chars The requested number of chars to skip
-    *
-    * @return The actual number of chars skipped.
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method skips the specified number of chars in the stream.  It
+   * returns the actual number of chars skipped, which may be less than the
+   * requested amount.
+   * <p>
+   * This method first discards chars in the buffer, then calls the
+   * <code>skip</code> method on the underlying stream to skip the 
+   * remaining chars.
+   *
+   * @param numChars The requested number of chars to skip
+   *
+   * @return The actual number of chars skipped.
+   *
+   * @exception IOException If an error occurs.
+   * @exception IllegalArgumentException If count is negative.
+   */
   public long skip(long count) throws IOException
   {
     synchronized (lock)
       {
 	checkStatus();
-	if (count <= 0)
+	if (count < 0)
+	  throw new IllegalArgumentException("skip value is negative");
+	if (count == 0)
 	  return 0;
 	// Yet again, we need to handle the special case of a readLine
 	// that has a '\r' at the end of the buffer.  In this case, we need
