@@ -1,5 +1,5 @@
 /* java.util.SimpleTimeZone
-   Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2003 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,8 +38,6 @@ exception statement from your version. */
 
 package java.util;
 
-import java.text.DateFormatSymbols;
-
 /**
  * This class represents a simple time zone offset and handles
  * daylight savings.  It can only handle one daylight savings rule, so
@@ -88,6 +86,7 @@ public class SimpleTimeZone extends TimeZone
   private static final int DOW_IN_MONTH_MODE = 2;
   private static final int DOW_GE_DOM_MODE = 3;
   private static final int DOW_LE_DOM_MODE = 4;
+  
   /**
    * The mode of the start rule. This takes one of the following values:
    * <dl>
@@ -391,7 +390,8 @@ public class SimpleTimeZone extends TimeZone
    * @param dayOfWeek The day of week where daylight savings start.
    * @param time The time in milliseconds standard time where daylight
    * savings start.
-   * @see SimpleTimeZone */
+   * @see SimpleTimeZone
+   */
   public void setStartRule(int month, int day, int dayOfWeek, int time)
   {
     this.startMode = checkRule(month, day, dayOfWeek);
@@ -412,11 +412,12 @@ public class SimpleTimeZone extends TimeZone
    *
    * @param rawOffset The time offset from GMT.
    * @param id  The identifier of this time zone.
-   * @param Month The end month of daylight savings.
+   * @param month The end month of daylight savings.
    * @param day A day in month, or a day of week in month.
-   * @param DayOfWeek A day of week, when daylight savings ends.
-   * @param Time A time in millis in standard time.
-   * @see #setStartRule */
+   * @param dayOfWeek A day of week, when daylight savings ends.
+   * @param time A time in millis in standard time.
+   * @see #setStartRule
+   */
   public void setEndRule(int month, int day, int dayOfWeek, int time)
   {
     this.endMode = checkRule(month, day, dayOfWeek);
@@ -488,7 +489,8 @@ public class SimpleTimeZone extends TimeZone
   /**
    * Returns the time zone offset to GMT in milliseconds, ignoring
    * day light savings.
-   * @return the time zone offset.  */
+   * @return the time zone offset.
+   */
   public int getRawOffset()
   {
     return rawOffset;
@@ -508,13 +510,30 @@ public class SimpleTimeZone extends TimeZone
    * milliseconds with respect to standard time.  Typically this
    * is one hour, but for some time zones this may be half an our.
    * @return the daylight savings offset in milliseconds.
-   * @since JDK1.1.4?
+   * 
+   * @since 1.2
    */
   public int getDSTSavings()
   {
     return dstSavings;
   }
 
+  /**
+   * Sets the daylight savings offset.  This is a positive offset in
+   * milliseconds with respect to standard time.
+   *
+   * @param dstSavings the daylight savings offset in milliseconds.
+   *
+   * @since 1.2
+   */
+  public void setDSTSavings(int dstSavings)
+  {
+    if (dstSavings <= 0)
+      throw new IllegalArgumentException("illegal value for dstSavings");
+    
+    this.dstSavings = dstSavings;
+  }
+
   /**
    * Returns if this time zone uses daylight savings time.
    * @return true, if we use daylight savings time, false otherwise.
